from enum import Enum


class LockReason(str, Enum):
    """Enumeration of reasons why a resource may be locked.

    Values indicate the cause for locking, such as a hard dependency or
    restricted access control preventing modifications.
    """

    HARDDEPENDENCY = "hardDependency"
    RESTRICTEDACCESS = "restrictedAccess"

    def __str__(self) -> str:
        return str(self.value)



from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from typing_extensions import Self

from templafy.types import UNSET, Unset

T = TypeVar("T", bound="UpdateLinkRequest")


@_attrs_define
class UpdateLinkRequest:
    """Attributes:
    folder_id (Union[None, Unset, int]): The identifier of the destination folder
    name (Union[None, Unset, str]): A new display name of the asset
    description (Union[None, Unset, str]): Description is used to specify the intended usage of the asset
    tags (Union[None, Unset, list[str]]): Tags should describe the content of the asset making it easier for a user
        to locate it
    url (Union[None, Unset, str]): A reference to the web resource. HTTP and HTTPS are supported
    external_data (Union[None, Unset, str]): External data which can be attached for future reference
    """

    folder_id: None | Unset | int = UNSET
    name: None | Unset | str = UNSET
    description: None | Unset | str = UNSET
    tags: None | Unset | list[str] = UNSET
    url: None | Unset | str = UNSET
    external_data: None | Unset | str = UNSET

    def to_dict(self) -> dict[str, Any]:
        folder_id: None | Unset | int
        if isinstance(self.folder_id, Unset):
            folder_id = UNSET
        else:
            folder_id = self.folder_id

        name: None | Unset | str
        if isinstance(self.name, Unset):
            name = UNSET
        else:
            name = self.name

        description: None | Unset | str
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description

        tags: None | Unset | list[str]
        if isinstance(self.tags, Unset):
            tags = UNSET
        elif isinstance(self.tags, list):
            tags = self.tags

        else:
            tags = self.tags

        url: None | Unset | str
        if isinstance(self.url, Unset):
            url = UNSET
        else:
            url = self.url

        external_data: None | Unset | str
        if isinstance(self.external_data, Unset):
            external_data = UNSET
        else:
            external_data = self.external_data

        field_dict: dict[str, Any] = {}

        field_dict.update({})
        if folder_id is not UNSET:
            field_dict["folderId"] = folder_id
        if name is not UNSET:
            field_dict["name"] = name
        if description is not UNSET:
            field_dict["description"] = description
        if tags is not UNSET:
            field_dict["tags"] = tags
        if url is not UNSET:
            field_dict["url"] = url
        if external_data is not UNSET:
            field_dict["externalData"] = external_data

        return field_dict

    @classmethod
    def from_dict(cls, src_dict: Mapping[str, Any]) -> Self:
        d = dict(src_dict)

        def _parse_folder_id(data: object) -> None | Unset | int:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast("None | Unset | int", data)

        folder_id = _parse_folder_id(d.pop("folderId", UNSET))

        def _parse_name(data: object) -> None | Unset | str:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast("None | Unset | str", data)

        name = _parse_name(d.pop("name", UNSET))

        def _parse_description(data: object) -> None | Unset | str:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast("None | Unset | str", data)

        description = _parse_description(d.pop("description", UNSET))

        def _parse_tags(data: object) -> None | Unset | list[str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError
                tags_type_0 = cast("list[str]", data)

                return tags_type_0
            except:  # noqa: E722
                pass
            return cast("None | Unset | list[str]", data)

        tags = _parse_tags(d.pop("tags", UNSET))

        def _parse_url(data: object) -> None | Unset | str:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast("None | Unset | str", data)

        url = _parse_url(d.pop("url", UNSET))

        def _parse_external_data(data: object) -> None | Unset | str:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast("None | Unset | str", data)

        external_data = _parse_external_data(d.pop("externalData", UNSET))

        update_link_request = cls(
            folder_id=folder_id,
            name=name,
            description=description,
            tags=tags,
            url=url,
            external_data=external_data,
        )

        return update_link_request



from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from typing_extensions import Self

from templafy.types import UNSET, Unset

T = TypeVar("T", bound="CreateDataSourceFieldSchemaRequest")


@_attrs_define
class CreateDataSourceFieldSchemaRequest:
    """Example:
        {'name': 'History', 'type': 'text', 'isMultipleLines': True, 'defaultValue': 'The city was established in the
            year 1652 by Dutch explorers...', 'isRequired': False}

    Attributes:
        type_ (str): Data source field schema type.
        name (str): The name of the field. It must be unique within the data source.
        is_required (Union[Unset, bool]): Whether the field is required. If true, the field must be filled in when
            creating a data source item.
    """

    type_: str
    name: str
    is_required: Unset | bool = UNSET

    def to_dict(self) -> dict[str, Any]:
        """Convert the object to a dictionary."""
        type_ = self.type_

        name = self.name

        is_required = self.is_required

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "type": type_,
                "name": name,
            }
        )
        if is_required is not UNSET:
            field_dict["isRequired"] = is_required

        return field_dict

    @classmethod
    def from_dict(cls, src_dict: Mapping[str, Any]) -> Self:
        """Create an instance from a dictionary."""
        d = dict(src_dict)
        type_ = d.pop("type")

        name = d.pop("name")

        is_required = d.pop("isRequired", UNSET)

        create_data_source_field_schema_request = cls(
            type_=type_,
            name=name,
            is_required=is_required,
        )

        return create_data_source_field_schema_request



from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from typing_extensions import Self

T = TypeVar("T", bound="CreateDataSourceItemFieldRequest")


@_attrs_define
class CreateDataSourceItemFieldRequest:
    """Attributes:
    type_ (str): Data source item field type.
    data_source_field_id (int): Data source field identifier.
    """

    type_: str
    data_source_field_id: int

    def to_dict(self) -> dict[str, Any]:
        """Convert the object to a dictionary."""
        type_ = self.type_

        data_source_field_id = self.data_source_field_id

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "type": type_,
                "dataSourceFieldId": data_source_field_id,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls, src_dict: Mapping[str, Any]) -> Self:
        """Create an instance from a dictionary."""
        d = dict(src_dict)
        type_ = d.pop("type")

        data_source_field_id = d.pop("dataSourceFieldId")

        create_data_source_item_field_request = cls(
            type_=type_,
            data_source_field_id=data_source_field_id,
        )

        return create_data_source_item_field_request



# ruff: noqa: E722
from http import HTTPStatus
from typing import Any, Union, cast

import httpx

from templafy import errors
from templafy.client import AuthenticatedClient, Client
from templafy.models.color_theme_field_schema import ColorThemeFieldSchema
from templafy.models.font_field_schema import FontFieldSchema
from templafy.models.image_field_schema import ImageFieldSchema
from templafy.models.language_field_schema import LanguageFieldSchema
from templafy.models.not_found_problem_details import NotFoundProblemDetails
from templafy.models.number_field_schema import NumberFieldSchema
from templafy.models.reference_field_schema import ReferenceFieldSchema
from templafy.models.text_field_schema import TextFieldSchema
from templafy.types import Response


def _get_kwargs(
    data_source_id: int,
    field_id: int,
) -> dict[str, Any]:
    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": f"/data-sources/{data_source_id}/fields/{field_id}",
    }

    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> (
    Any
    | NotFoundProblemDetails
    | Union[
        "ColorThemeFieldSchema",
        "FontFieldSchema",
        "ImageFieldSchema",
        "LanguageFieldSchema",
        "NumberFieldSchema",
        "ReferenceFieldSchema",
        "TextFieldSchema",
    ]
    | None
):
    if response.status_code == 200:

        def _parse_response_200(
            data: object,
        ) -> Union[
            "ColorThemeFieldSchema",
            "FontFieldSchema",
            "ImageFieldSchema",
            "LanguageFieldSchema",
            "NumberFieldSchema",
            "ReferenceFieldSchema",
            "TextFieldSchema",
        ]:
            try:
                if not isinstance(data, dict):
                    raise TypeError
                response_200_type_0 = TextFieldSchema.from_dict(data)

                return response_200_type_0
            except:
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError
                response_200_type_1 = NumberFieldSchema.from_dict(data)

                return response_200_type_1
            except:
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError
                response_200_type_2 = ReferenceFieldSchema.from_dict(data)

                return response_200_type_2
            except:
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError
                response_200_type_3 = ImageFieldSchema.from_dict(data)

                return response_200_type_3
            except:
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError
                response_200_type_4 = LanguageFieldSchema.from_dict(data)

                return response_200_type_4
            except:
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError
                response_200_type_5 = FontFieldSchema.from_dict(data)

                return response_200_type_5
            except:
                pass
            if not isinstance(data, dict):
                raise TypeError
            response_200_type_6 = ColorThemeFieldSchema.from_dict(data)

            return response_200_type_6

        response_200 = _parse_response_200(response.json())

        return response_200

    if response.status_code == 401:
        response_401 = cast("Any", None)
        return response_401

    if response.status_code == 403:
        response_403 = cast("Any", None)
        return response_403

    if response.status_code == 404:
        response_404 = NotFoundProblemDetails.from_dict(response.json())

        return response_404

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[
    Any
    | NotFoundProblemDetails
    | Union[
        "ColorThemeFieldSchema",
        "FontFieldSchema",
        "ImageFieldSchema",
        "LanguageFieldSchema",
        "NumberFieldSchema",
        "ReferenceFieldSchema",
        "TextFieldSchema",
    ]
]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    data_source_id: int,
    field_id: int,
    *,
    client: AuthenticatedClient,
) -> Response[
    Any
    | NotFoundProblemDetails
    | Union[
        "ColorThemeFieldSchema",
        "FontFieldSchema",
        "ImageFieldSchema",
        "LanguageFieldSchema",
        "NumberFieldSchema",
        "ReferenceFieldSchema",
        "TextFieldSchema",
    ]
]:
    """Gets an existing data source field.

     Returns a data source field that represents the schema of a single field of a data source.

    Args:
        data_source_id (int): The ID of the data source.
        field_id (int): The ID of the field.
        client (AuthenticatedClient): The authenticated client instance.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, NotFoundProblemDetails, Union['ColorThemeFieldSchema', 'FontFieldSchema', 'ImageFieldSchema', 'LanguageFieldSchema', 'NumberFieldSchema', 'ReferenceFieldSchema', 'TextFieldSchema']]]
    """

    kwargs = _get_kwargs(
        data_source_id=data_source_id,
        field_id=field_id,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    data_source_id: int,
    field_id: int,
    *,
    client: AuthenticatedClient,
) -> (
    Any
    | NotFoundProblemDetails
    | Union[
        "ColorThemeFieldSchema",
        "FontFieldSchema",
        "ImageFieldSchema",
        "LanguageFieldSchema",
        "NumberFieldSchema",
        "ReferenceFieldSchema",
        "TextFieldSchema",
    ]
    | None
):
    """Gets an existing data source field.

     Returns a data source field that represents the schema of a single field of a data source.

    Args:
        data_source_id (int): The ID of the data source.
        field_id (int): The ID of the field.
        client (AuthenticatedClient): The authenticated client instance.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, NotFoundProblemDetails, Union['ColorThemeFieldSchema', 'FontFieldSchema', 'ImageFieldSchema', 'LanguageFieldSchema', 'NumberFieldSchema', 'ReferenceFieldSchema', 'TextFieldSchema']]
    """

    return sync_detailed(
        data_source_id=data_source_id,
        field_id=field_id,
        client=client,
    ).parsed


async def asyncio_detailed(
    data_source_id: int,
    field_id: int,
    *,
    client: AuthenticatedClient,
) -> Response[
    Any
    | NotFoundProblemDetails
    | Union[
        "ColorThemeFieldSchema",
        "FontFieldSchema",
        "ImageFieldSchema",
        "LanguageFieldSchema",
        "NumberFieldSchema",
        "ReferenceFieldSchema",
        "TextFieldSchema",
    ]
]:
    """Gets an existing data source field.

     Returns a data source field that represents the schema of a single field of a data source.

    Args:
        data_source_id (int): The ID of the data source.
        field_id (int): The ID of the field.
        client (AuthenticatedClient): The authenticated client instance.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, NotFoundProblemDetails, Union['ColorThemeFieldSchema', 'FontFieldSchema', 'ImageFieldSchema', 'LanguageFieldSchema', 'NumberFieldSchema', 'ReferenceFieldSchema', 'TextFieldSchema']]]
    """

    kwargs = _get_kwargs(
        data_source_id=data_source_id,
        field_id=field_id,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    data_source_id: int,
    field_id: int,
    *,
    client: AuthenticatedClient,
) -> (
    Any
    | NotFoundProblemDetails
    | Union[
        "ColorThemeFieldSchema",
        "FontFieldSchema",
        "ImageFieldSchema",
        "LanguageFieldSchema",
        "NumberFieldSchema",
        "ReferenceFieldSchema",
        "TextFieldSchema",
    ]
    | None
):
    """Gets an existing data source field.

     Returns a data source field that represents the schema of a single field of a data source.

    Args:
        data_source_id (int): The ID of the data source.
        field_id (int): The ID of the field.
        client (AuthenticatedClient): The authenticated client instance.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, NotFoundProblemDetails, Union['ColorThemeFieldSchema', 'FontFieldSchema', 'ImageFieldSchema', 'LanguageFieldSchema', 'NumberFieldSchema', 'ReferenceFieldSchema', 'TextFieldSchema']]
    """

    return (
        await asyncio_detailed(
            data_source_id=data_source_id,
            field_id=field_id,
            client=client,
        )
    ).parsed



from http import HTTPStatus
from typing import Any, cast

import httpx

from templafy import errors
from templafy.client import AuthenticatedClient, Client
from templafy.models.conflict_problem_details import ConflictProblemDetails
from templafy.models.not_found_problem_details import NotFoundProblemDetails
from templafy.models.patch_libraries_space_id_spreadsheets_assets_asset_id_body import (
    PatchLibrariesSpaceIdSpreadsheetsAssetsAssetIdBody,
)
from templafy.models.validation_problem_details import ValidationProblemDetails
from templafy.types import Response


def _get_kwargs(
    space_id: int,
    asset_id: int,
    *,
    body: PatchLibrariesSpaceIdSpreadsheetsAssetsAssetIdBody,
) -> dict[str, Any]:
    headers: dict[str, Any] = {}

    _kwargs: dict[str, Any] = {
        "method": "patch",
        "url": f"/libraries/{space_id}/spreadsheets/assets/{asset_id}",
    }

    _kwargs["files"] = body.to_multipart()

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> (
    Any
    | ConflictProblemDetails
    | NotFoundProblemDetails
    | ValidationProblemDetails
    | None
):
    if response.status_code == 204:
        response_204 = cast("Any", None)
        return response_204

    if response.status_code == 400:
        response_400 = ValidationProblemDetails.from_dict(response.json())

        return response_400

    if response.status_code == 401:
        response_401 = cast("Any", None)
        return response_401

    if response.status_code == 403:
        response_403 = cast("Any", None)
        return response_403

    if response.status_code == 404:
        response_404 = NotFoundProblemDetails.from_dict(response.json())

        return response_404

    if response.status_code == 409:
        response_409 = ConflictProblemDetails.from_dict(response.json())

        return response_409

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[
    Any | ConflictProblemDetails | NotFoundProblemDetails | ValidationProblemDetails
]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    space_id: int,
    asset_id: int,
    *,
    client: AuthenticatedClient,
    body: PatchLibrariesSpaceIdSpreadsheetsAssetsAssetIdBody,
) -> Response[
    Any | ConflictProblemDetails | NotFoundProblemDetails | ValidationProblemDetails
]:
    """Updates the spreadsheet template.

     Only one file can be attached to the request body. The supported file formats are .XLSX and .XLSM
    for macro-enabled spreadsheets.

    Args:
        space_id (int):
        asset_id (int):
        body (PatchLibrariesSpaceIdSpreadsheetsAssetsAssetIdBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, ConflictProblemDetails, NotFoundProblemDetails, ValidationProblemDetails]]
    """

    kwargs = _get_kwargs(
        space_id=space_id,
        asset_id=asset_id,
        body=body,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    space_id: int,
    asset_id: int,
    *,
    client: AuthenticatedClient,
    body: PatchLibrariesSpaceIdSpreadsheetsAssetsAssetIdBody,
) -> (
    Any
    | ConflictProblemDetails
    | NotFoundProblemDetails
    | ValidationProblemDetails
    | None
):
    """Updates the spreadsheet template.

     Only one file can be attached to the request body. The supported file formats are .XLSX and .XLSM
    for macro-enabled spreadsheets.

    Args:
        space_id (int):
        asset_id (int):
        body (PatchLibrariesSpaceIdSpreadsheetsAssetsAssetIdBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, ConflictProblemDetails, NotFoundProblemDetails, ValidationProblemDetails]
    """

    return sync_detailed(
        space_id=space_id,
        asset_id=asset_id,
        client=client,
        body=body,
    ).parsed


async def asyncio_detailed(
    space_id: int,
    asset_id: int,
    *,
    client: AuthenticatedClient,
    body: PatchLibrariesSpaceIdSpreadsheetsAssetsAssetIdBody,
) -> Response[
    Any | ConflictProblemDetails | NotFoundProblemDetails | ValidationProblemDetails
]:
    """Updates the spreadsheet template.

     Only one file can be attached to the request body. The supported file formats are .XLSX and .XLSM
    for macro-enabled spreadsheets.

    Args:
        space_id (int):
        asset_id (int):
        body (PatchLibrariesSpaceIdSpreadsheetsAssetsAssetIdBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, ConflictProblemDetails, NotFoundProblemDetails, ValidationProblemDetails]]
    """

    kwargs = _get_kwargs(
        space_id=space_id,
        asset_id=asset_id,
        body=body,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    space_id: int,
    asset_id: int,
    *,
    client: AuthenticatedClient,
    body: PatchLibrariesSpaceIdSpreadsheetsAssetsAssetIdBody,
) -> (
    Any
    | ConflictProblemDetails
    | NotFoundProblemDetails
    | ValidationProblemDetails
    | None
):
    """Updates the spreadsheet template.

     Only one file can be attached to the request body. The supported file formats are .XLSX and .XLSM
    for macro-enabled spreadsheets.

    Args:
        space_id (int):
        asset_id (int):
        body (PatchLibrariesSpaceIdSpreadsheetsAssetsAssetIdBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, ConflictProblemDetails, NotFoundProblemDetails, ValidationProblemDetails]
    """

    return (
        await asyncio_detailed(
            space_id=space_id,
            asset_id=asset_id,
            client=client,
            body=body,
        )
    ).parsed



from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from typing_extensions import Self

from templafy.types import UNSET, Unset

T = TypeVar("T", bound="UpdateFolderRequest")


@_attrs_define
class UpdateFolderRequest:
    """The request model to update the folder

    Attributes:
        name (Union[None, Unset, str]): Display name
        parent_folder_id (Union[None, Unset, int]): The identifier of a folder that current folder should be moved to
    """

    name: None | Unset | str = UNSET
    parent_folder_id: None | Unset | int = UNSET

    def to_dict(self) -> dict[str, Any]:
        name: None | Unset | str
        if isinstance(self.name, Unset):
            name = UNSET
        else:
            name = self.name

        parent_folder_id: None | Unset | int
        if isinstance(self.parent_folder_id, Unset):
            parent_folder_id = UNSET
        else:
            parent_folder_id = self.parent_folder_id

        field_dict: dict[str, Any] = {}

        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if parent_folder_id is not UNSET:
            field_dict["parentFolderId"] = parent_folder_id

        return field_dict

    @classmethod
    def from_dict(cls, src_dict: Mapping[str, Any]) -> Self:
        d = dict(src_dict)

        def _parse_name(data: object) -> None | Unset | str:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast("None | Unset | str", data)

        name = _parse_name(d.pop("name", UNSET))

        def _parse_parent_folder_id(data: object) -> None | Unset | int:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast("None | Unset | int", data)

        parent_folder_id = _parse_parent_folder_id(d.pop("parentFolderId", UNSET))

        update_folder_request = cls(
            name=name,
            parent_folder_id=parent_folder_id,
        )

        return update_folder_request



from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from typing_extensions import Self

from templafy.types import UNSET, Unset

T = TypeVar("T", bound="DataSourceItemField")


@_attrs_define
class DataSourceItemField:
    """Attributes:
    type_ (Union[None, str]): Data source item field type.
    data_source_field_id (Union[Unset, int]): Unique data source field identifier.
    """

    type_: None | str
    data_source_field_id: Unset | int = UNSET

    def to_dict(self) -> dict[str, Any]:
        type_: None | str
        type_ = self.type_

        data_source_field_id = self.data_source_field_id

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "type": type_,
            }
        )
        if data_source_field_id is not UNSET:
            field_dict["dataSourceFieldId"] = data_source_field_id

        return field_dict

    @classmethod
    def from_dict(cls, src_dict: Mapping[str, Any]) -> Self:
        d = dict(src_dict)

        def _parse_type_(data: object) -> None | str:
            if data is None:
                return data
            return cast("None | str", data)

        type_ = _parse_type_(d.pop("type"))

        data_source_field_id = d.pop("dataSourceFieldId", UNSET)

        data_source_item_field = cls(
            type_=type_,
            data_source_field_id=data_source_field_id,
        )

        return data_source_item_field



from http import HTTPStatus
from typing import Any, cast

import httpx

from templafy import errors
from templafy.client import AuthenticatedClient, Client
from templafy.models.create_data_source_request import CreateDataSourceRequest
from templafy.models.data_source import DataSource
from templafy.models.not_found_problem_details import NotFoundProblemDetails
from templafy.models.validation_problem_details import ValidationProblemDetails
from templafy.types import Response


def _get_kwargs(
    *,
    body: CreateDataSourceRequest,
) -> dict[str, Any]:
    headers: dict[str, Any] = {}

    _kwargs: dict[str, Any] = {
        "method": "post",
        "url": "/data-sources",
    }

    if isinstance(body, CreateDataSourceRequest):
        _kwargs["json"] = body.to_dict()

        headers["Content-Type"] = "application/json-patch+json"
    if isinstance(body, CreateDataSourceRequest):
        _kwargs["json"] = body.to_dict()

        headers["Content-Type"] = "application/json"
    if isinstance(body, CreateDataSourceRequest):
        _kwargs["json"] = body.to_dict()

        headers["Content-Type"] = "application/*+json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Any | DataSource | NotFoundProblemDetails | ValidationProblemDetails | None:
    if response.status_code == 201:
        response_201 = DataSource.from_dict(response.json())

        return response_201

    if response.status_code == 400:
        response_400 = ValidationProblemDetails.from_dict(response.json())

        return response_400

    if response.status_code == 401:
        response_401 = cast("Any", None)
        return response_401

    if response.status_code == 403:
        response_403 = cast("Any", None)
        return response_403

    if response.status_code == 404:
        response_404 = NotFoundProblemDetails.from_dict(response.json())

        return response_404

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[Any | DataSource | NotFoundProblemDetails | ValidationProblemDetails]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    body: CreateDataSourceRequest,
) -> Response[Any | DataSource | NotFoundProblemDetails | ValidationProblemDetails]:
    """Creates a new data source.

     Creates a new data source to contain data source items to represent your data.

    Args:
        body (CreateDataSourceRequest):  Example: {'name': 'Cities', 'description': 'Cities in
            which we have offices', 'fields': [{'name': 'History', 'type': 'text', 'isMultipleLines':
            True, 'defaultValue': 'The city was established in the year 1652 by Dutch explorers...',
            'isRequired': False}, {'name': 'Population', 'type': 'number', 'defaultValue': 100222,
            'isRequired': True}, {'name': 'Country', 'type': 'reference', 'referenceDataSourceId':
            '637989101951089955', 'defaultReferencedItemId': '638249311425155568', 'isRequired':
            True}, {'name': 'Flag', 'type': 'image', 'isRequired': True}]}.
        body (CreateDataSourceRequest):  Example: {'name': 'Cities', 'description': 'Cities in
            which we have offices', 'fields': [{'name': 'History', 'type': 'text', 'isMultipleLines':
            True, 'defaultValue': 'The city was established in the year 1652 by Dutch explorers...',
            'isRequired': False}, {'name': 'Population', 'type': 'number', 'defaultValue': 100222,
            'isRequired': True}, {'name': 'Country', 'type': 'reference', 'referenceDataSourceId':
            '637989101951089955', 'defaultReferencedItemId': '638249311425155568', 'isRequired':
            True}, {'name': 'Flag', 'type': 'image', 'isRequired': True}]}.
        body (CreateDataSourceRequest):  Example: {'name': 'Cities', 'description': 'Cities in
            which we have offices', 'fields': [{'name': 'History', 'type': 'text', 'isMultipleLines':
            True, 'defaultValue': 'The city was established in the year 1652 by Dutch explorers...',
            'isRequired': False}, {'name': 'Population', 'type': 'number', 'defaultValue': 100222,
            'isRequired': True}, {'name': 'Country', 'type': 'reference', 'referenceDataSourceId':
            '637989101951089955', 'defaultReferencedItemId': '638249311425155568', 'isRequired':
            True}, {'name': 'Flag', 'type': 'image', 'isRequired': True}]}.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, DataSource, NotFoundProblemDetails, ValidationProblemDetails]]
    """

    kwargs = _get_kwargs(
        body=body,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    body: CreateDataSourceRequest,
) -> Any | DataSource | NotFoundProblemDetails | ValidationProblemDetails | None:
    """Creates a new data source.

     Creates a new data source to contain data source items to represent your data.

    Args:
        body (CreateDataSourceRequest):  Example: {'name': 'Cities', 'description': 'Cities in
            which we have offices', 'fields': [{'name': 'History', 'type': 'text', 'isMultipleLines':
            True, 'defaultValue': 'The city was established in the year 1652 by Dutch explorers...',
            'isRequired': False}, {'name': 'Population', 'type': 'number', 'defaultValue': 100222,
            'isRequired': True}, {'name': 'Country', 'type': 'reference', 'referenceDataSourceId':
            '637989101951089955', 'defaultReferencedItemId': '638249311425155568', 'isRequired':
            True}, {'name': 'Flag', 'type': 'image', 'isRequired': True}]}.
        body (CreateDataSourceRequest):  Example: {'name': 'Cities', 'description': 'Cities in
            which we have offices', 'fields': [{'name': 'History', 'type': 'text', 'isMultipleLines':
            True, 'defaultValue': 'The city was established in the year 1652 by Dutch explorers...',
            'isRequired': False}, {'name': 'Population', 'type': 'number', 'defaultValue': 100222,
            'isRequired': True}, {'name': 'Country', 'type': 'reference', 'referenceDataSourceId':
            '637989101951089955', 'defaultReferencedItemId': '638249311425155568', 'isRequired':
            True}, {'name': 'Flag', 'type': 'image', 'isRequired': True}]}.
        body (CreateDataSourceRequest):  Example: {'name': 'Cities', 'description': 'Cities in
            which we have offices', 'fields': [{'name': 'History', 'type': 'text', 'isMultipleLines':
            True, 'defaultValue': 'The city was established in the year 1652 by Dutch explorers...',
            'isRequired': False}, {'name': 'Population', 'type': 'number', 'defaultValue': 100222,
            'isRequired': True}, {'name': 'Country', 'type': 'reference', 'referenceDataSourceId':
            '637989101951089955', 'defaultReferencedItemId': '638249311425155568', 'isRequired':
            True}, {'name': 'Flag', 'type': 'image', 'isRequired': True}]}.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, DataSource, NotFoundProblemDetails, ValidationProblemDetails]
    """

    return sync_detailed(
        client=client,
        body=body,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    body: CreateDataSourceRequest,
) -> Response[Any | DataSource | NotFoundProblemDetails | ValidationProblemDetails]:
    """Creates a new data source.

     Creates a new data source to contain data source items to represent your data.

    Args:
        body (CreateDataSourceRequest):  Example: {'name': 'Cities', 'description': 'Cities in
            which we have offices', 'fields': [{'name': 'History', 'type': 'text', 'isMultipleLines':
            True, 'defaultValue': 'The city was established in the year 1652 by Dutch explorers...',
            'isRequired': False}, {'name': 'Population', 'type': 'number', 'defaultValue': 100222,
            'isRequired': True}, {'name': 'Country', 'type': 'reference', 'referenceDataSourceId':
            '637989101951089955', 'defaultReferencedItemId': '638249311425155568', 'isRequired':
            True}, {'name': 'Flag', 'type': 'image', 'isRequired': True}]}.
        body (CreateDataSourceRequest):  Example: {'name': 'Cities', 'description': 'Cities in
            which we have offices', 'fields': [{'name': 'History', 'type': 'text', 'isMultipleLines':
            True, 'defaultValue': 'The city was established in the year 1652 by Dutch explorers...',
            'isRequired': False}, {'name': 'Population', 'type': 'number', 'defaultValue': 100222,
            'isRequired': True}, {'name': 'Country', 'type': 'reference', 'referenceDataSourceId':
            '637989101951089955', 'defaultReferencedItemId': '638249311425155568', 'isRequired':
            True}, {'name': 'Flag', 'type': 'image', 'isRequired': True}]}.
        body (CreateDataSourceRequest):  Example: {'name': 'Cities', 'description': 'Cities in
            which we have offices', 'fields': [{'name': 'History', 'type': 'text', 'isMultipleLines':
            True, 'defaultValue': 'The city was established in the year 1652 by Dutch explorers...',
            'isRequired': False}, {'name': 'Population', 'type': 'number', 'defaultValue': 100222,
            'isRequired': True}, {'name': 'Country', 'type': 'reference', 'referenceDataSourceId':
            '637989101951089955', 'defaultReferencedItemId': '638249311425155568', 'isRequired':
            True}, {'name': 'Flag', 'type': 'image', 'isRequired': True}]}.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, DataSource, NotFoundProblemDetails, ValidationProblemDetails]]
    """

    kwargs = _get_kwargs(
        body=body,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    body: CreateDataSourceRequest,
) -> Any | DataSource | NotFoundProblemDetails | ValidationProblemDetails | None:
    """Creates a new data source.

     Creates a new data source to contain data source items to represent your data.

    Args:
        body (CreateDataSourceRequest):  Example: {'name': 'Cities', 'description': 'Cities in
            which we have offices', 'fields': [{'name': 'History', 'type': 'text', 'isMultipleLines':
            True, 'defaultValue': 'The city was established in the year 1652 by Dutch explorers...',
            'isRequired': False}, {'name': 'Population', 'type': 'number', 'defaultValue': 100222,
            'isRequired': True}, {'name': 'Country', 'type': 'reference', 'referenceDataSourceId':
            '637989101951089955', 'defaultReferencedItemId': '638249311425155568', 'isRequired':
            True}, {'name': 'Flag', 'type': 'image', 'isRequired': True}]}.
        body (CreateDataSourceRequest):  Example: {'name': 'Cities', 'description': 'Cities in
            which we have offices', 'fields': [{'name': 'History', 'type': 'text', 'isMultipleLines':
            True, 'defaultValue': 'The city was established in the year 1652 by Dutch explorers...',
            'isRequired': False}, {'name': 'Population', 'type': 'number', 'defaultValue': 100222,
            'isRequired': True}, {'name': 'Country', 'type': 'reference', 'referenceDataSourceId':
            '637989101951089955', 'defaultReferencedItemId': '638249311425155568', 'isRequired':
            True}, {'name': 'Flag', 'type': 'image', 'isRequired': True}]}.
        body (CreateDataSourceRequest):  Example: {'name': 'Cities', 'description': 'Cities in
            which we have offices', 'fields': [{'name': 'History', 'type': 'text', 'isMultipleLines':
            True, 'defaultValue': 'The city was established in the year 1652 by Dutch explorers...',
            'isRequired': False}, {'name': 'Population', 'type': 'number', 'defaultValue': 100222,
            'isRequired': True}, {'name': 'Country', 'type': 'reference', 'referenceDataSourceId':
            '637989101951089955', 'defaultReferencedItemId': '638249311425155568', 'isRequired':
            True}, {'name': 'Flag', 'type': 'image', 'isRequired': True}]}.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, DataSource, NotFoundProblemDetails, ValidationProblemDetails]
    """

    return (
        await asyncio_detailed(
            client=client,
            body=body,
        )
    ).parsed



from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from typing_extensions import Self

from templafy.types import UNSET, Unset

T = TypeVar("T", bound="Dimensions")


@_attrs_define
class Dimensions:
    """Asset dimensions

    Attributes:
        height (int): Height of the asset in pixels
        width (int): Width of the asset in pixels
        aspect_ratio (Union[None, Unset, str]):
    """

    height: int
    width: int
    aspect_ratio: None | Unset | str = UNSET

    def to_dict(self) -> dict[str, Any]:
        height = self.height

        width = self.width

        aspect_ratio: None | Unset | str
        if isinstance(self.aspect_ratio, Unset):
            aspect_ratio = UNSET
        else:
            aspect_ratio = self.aspect_ratio

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "height": height,
                "width": width,
            }
        )
        if aspect_ratio is not UNSET:
            field_dict["aspectRatio"] = aspect_ratio

        return field_dict

    @classmethod
    def from_dict(cls, src_dict: Mapping[str, Any]) -> Self:
        d = dict(src_dict)
        height = d.pop("height")

        width = d.pop("width")

        def _parse_aspect_ratio(data: object) -> None | Unset | str:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast("None | Unset | str", data)

        aspect_ratio = _parse_aspect_ratio(d.pop("aspectRatio", UNSET))

        dimensions = cls(
            height=height,
            width=width,
            aspect_ratio=aspect_ratio,
        )

        return dimensions



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from typing_extensions import Self

from templafy.types import UNSET, Unset

if TYPE_CHECKING:
    from templafy.models.create_color_theme_data_source_item_field_request import (
        CreateColorThemeDataSourceItemFieldRequest,
    )
    from templafy.models.create_font_data_source_item_field_request import (
        CreateFontDataSourceItemFieldRequest,
    )
    from templafy.models.create_image_data_source_item_field_request import (
        CreateImageDataSourceItemFieldRequest,
    )
    from templafy.models.create_number_data_source_item_field_request import (
        CreateNumberDataSourceItemFieldRequest,
    )
    from templafy.models.create_reference_data_source_item_field_request import (
        CreateReferenceDataSourceItemFieldRequest,
    )
    from templafy.models.create_text_data_source_item_field_request import (
        CreateTextDataSourceItemFieldRequest,
    )
from templafy.models.create_color_theme_data_source_item_field_request import (
    CreateColorThemeDataSourceItemFieldRequest,
)
from templafy.models.create_font_data_source_item_field_request import (
    CreateFontDataSourceItemFieldRequest,
)
from templafy.models.create_image_data_source_item_field_request import (
    CreateImageDataSourceItemFieldRequest,
)
from templafy.models.create_number_data_source_item_field_request import (
    CreateNumberDataSourceItemFieldRequest,
)
from templafy.models.create_reference_data_source_item_field_request import (
    CreateReferenceDataSourceItemFieldRequest,
)
from templafy.models.create_text_data_source_item_field_request import (
    CreateTextDataSourceItemFieldRequest,
)

T = TypeVar("T", bound="CreateDataSourceItemRequest")


@_attrs_define
class CreateDataSourceItemRequest:
    """Example:
        {'fields': [{'dataSourceFieldId': 0, 'type': 'text', 'value': 'Sample text'}, {'dataSourceFieldId': 1, 'type':
            'number', 'value': 123.45}, {'dataSourceFieldId': 2, 'type': 'reference', 'dataSourceItemId':
            '638247997437572264'}, {'dataSourceFieldId': 3, 'type': 'image', 'fileName': 'Cat', 'fileUrl':
            'https://en.wikipedia.org/wiki/Cat#/media/File:Cat_August_2010-4.jpg'}, {'dataSourceFieldId': 4, 'type': 'font',
            'fileName': 'best-font', 'fileUrl': 'https://allfonts.com/best-font'}, {'dataSourceFieldId': 5, 'type':
            'colorTheme', 'xmlValue': '<a:clrScheme xmlns:a="http://schemas.openxmlformats.org/drawingml/2006/main"
            name="Templafy_New"><a:dk1><a:srgbClr val="0078FF"/></a:dk1><a:lt1><a:sysClr val="window"
            lastClr="FFFFFF"/></a:lt1><a:dk2><a:srgbClr val="000000"/></a:dk2><a:lt2><a:srgbClr
            val="E5E5E5"/></a:lt2><a:accent1><a:srgbClr val="0078FF"/></a:accent1><a:accent2><a:srgbClr
            val="575757"/></a:accent2><a:accent3><a:srgbClr val="12AA96"/></a:accent3><a:accent4><a:srgbClr
            val="15436B"/></a:accent4><a:accent5><a:srgbClr val="D44849"/></a:accent5><a:accent6><a:srgbClr
            val="7F7F7F"/></a:accent6><a:hlink><a:srgbClr val="0078FF"/></a:hlink><a:folHlink><a:srgbClr
            val="55CBFF"/></a:folHlink></a:clrScheme>'}]}

    Attributes:
        fields (Union[None, Unset, list[Union['CreateColorThemeDataSourceItemFieldRequest',
            'CreateFontDataSourceItemFieldRequest', 'CreateImageDataSourceItemFieldRequest',
            'CreateNumberDataSourceItemFieldRequest', 'CreateReferenceDataSourceItemFieldRequest',
            'CreateTextDataSourceItemFieldRequest']]]): The fields of the data source item.
    """

    fields: (
        None
        | Unset
        | list[
            Union[
                "CreateColorThemeDataSourceItemFieldRequest",
                "CreateFontDataSourceItemFieldRequest",
                "CreateImageDataSourceItemFieldRequest",
                "CreateNumberDataSourceItemFieldRequest",
                "CreateReferenceDataSourceItemFieldRequest",
                "CreateTextDataSourceItemFieldRequest",
            ]
        ]
    ) = UNSET

    def to_dict(self) -> dict[str, Any]:
        """Convert the object to a dictionary."""
        # Top-level imports are used to avoid inline imports (PLC0415)

        fields: None | Unset | list[dict[str, Any]]
        if isinstance(self.fields, Unset):
            fields = UNSET
        elif isinstance(self.fields, list):
            fields = []
            for fields_type_0_item_data in self.fields:
                fields_type_0_item: dict[str, Any]
                if isinstance(
                    fields_type_0_item_data,
                    (
                        CreateTextDataSourceItemFieldRequest,
                        CreateNumberDataSourceItemFieldRequest,
                        CreateReferenceDataSourceItemFieldRequest,
                        CreateImageDataSourceItemFieldRequest,
                        CreateFontDataSourceItemFieldRequest,
                    ),
                ):
                    fields_type_0_item = fields_type_0_item_data.to_dict()
                else:
                    fields_type_0_item = fields_type_0_item_data.to_dict()

                fields.append(fields_type_0_item)

        else:
            fields = self.fields

        field_dict: dict[str, Any] = {}

        field_dict.update({})
        if fields is not UNSET:
            field_dict["fields"] = fields

        return field_dict

    @classmethod
    def from_dict(cls, src_dict: Mapping[str, Any]) -> Self:
        """Create an instance from a dictionary."""
        # Top-level imports are used to avoid inline imports (PLC0415)

        d = dict(src_dict)

        def _parse_fields(
            data: object,
        ) -> (
            None
            | Unset
            | list[
                Union[
                    "CreateColorThemeDataSourceItemFieldRequest",
                    "CreateFontDataSourceItemFieldRequest",
                    "CreateImageDataSourceItemFieldRequest",
                    "CreateNumberDataSourceItemFieldRequest",
                    "CreateReferenceDataSourceItemFieldRequest",
                    "CreateTextDataSourceItemFieldRequest",
                ]
            ]
        ):
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError
                fields_type_0 = []
                _fields_type_0 = data
                for fields_type_0_item_data in _fields_type_0:

                    def _parse_fields_type_0_item(
                        data: object,
                    ) -> Union[
                        "CreateColorThemeDataSourceItemFieldRequest",
                        "CreateFontDataSourceItemFieldRequest",
                        "CreateImageDataSourceItemFieldRequest",
                        "CreateNumberDataSourceItemFieldRequest",
                        "CreateReferenceDataSourceItemFieldRequest",
                        "CreateTextDataSourceItemFieldRequest",
                    ]:
                        try:
                            if not isinstance(data, dict):
                                raise TypeError
                            fields_type_0_item_type_0 = (
                                CreateTextDataSourceItemFieldRequest.from_dict(data)
                            )

                            return fields_type_0_item_type_0
                        except:  # noqa: E722
                            pass
                        try:
                            if not isinstance(data, dict):
                                raise TypeError
                            fields_type_0_item_type_1 = (
                                CreateNumberDataSourceItemFieldRequest.from_dict(data)
                            )

                            return fields_type_0_item_type_1
                        except:  # noqa: E722
                            pass
                        try:
                            if not isinstance(data, dict):
                                raise TypeError
                            fields_type_0_item_type_2 = (
                                CreateReferenceDataSourceItemFieldRequest.from_dict(
                                    data
                                )
                            )

                            return fields_type_0_item_type_2
                        except:  # noqa: E722
                            pass
                        try:
                            if not isinstance(data, dict):
                                raise TypeError
                            fields_type_0_item_type_3 = (
                                CreateImageDataSourceItemFieldRequest.from_dict(data)
                            )

                            return fields_type_0_item_type_3
                        except:  # noqa: E722
                            pass
                        try:
                            if not isinstance(data, dict):
                                raise TypeError
                            fields_type_0_item_type_4 = (
                                CreateFontDataSourceItemFieldRequest.from_dict(data)
                            )

                            return fields_type_0_item_type_4
                        except:  # noqa: E722
                            pass
                        if not isinstance(data, dict):
                            raise TypeError
                        fields_type_0_item_type_5 = (
                            CreateColorThemeDataSourceItemFieldRequest.from_dict(data)
                        )

                        return fields_type_0_item_type_5

                    fields_type_0_item = _parse_fields_type_0_item(
                        fields_type_0_item_data
                    )

                    fields_type_0.append(fields_type_0_item)

                return fields_type_0
            except:  # noqa: E722
                pass
            return cast(
                "None | Unset | list[CreateColorThemeDataSourceItemFieldRequest | CreateFontDataSourceItemFieldRequest | CreateImageDataSourceItemFieldRequest | CreateNumberDataSourceItemFieldRequest | CreateReferenceDataSourceItemFieldRequest | CreateTextDataSourceItemFieldRequest]",
                data,
            )

        fields = _parse_fields(d.pop("fields", UNSET))

        create_data_source_item_request = cls(
            fields=fields,
        )

        return create_data_source_item_request



from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Self

T = TypeVar("T", bound="ValidationProblemDetailsErrorsType0")


@_attrs_define
class ValidationProblemDetailsErrorsType0:
    """Mapping of validation error fields to lists of error messages."""

    additional_properties: dict[str, list[str]] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        field_dict: dict[str, Any] = dict(self.additional_properties)

        return field_dict

    @classmethod
    def from_dict(cls, src_dict: Mapping[str, Any]) -> Self:
        d = dict(src_dict)
        validation_problem_details_errors_type_0 = cls()

        additional_properties = {
            prop_name: cast("list[str]", prop_dict)
            for prop_name, prop_dict in d.items()
        }

        validation_problem_details_errors_type_0.additional_properties = (
            additional_properties
        )
        return validation_problem_details_errors_type_0

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> list[str]:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: list[str]) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from typing_extensions import Self

from templafy.models.library_type import LibraryType

T = TypeVar("T", bound="Library")


@_attrs_define
class Library:
    """Attributes:
    id (int): Unique library identifier
    name (str): Display name of the library
    library_type (LibraryType): Type of the assets that can be stored in the library
    space_id (int): Unique identifier of the space to which the library belongs
    root_folder_id (int): Unique identifier of the root folder of the library
    """

    id: int
    name: str
    library_type: LibraryType
    space_id: int
    root_folder_id: int

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        name = self.name

        library_type = self.library_type.value

        space_id = self.space_id

        root_folder_id = self.root_folder_id

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "id": id,
                "name": name,
                "libraryType": library_type,
                "spaceId": space_id,
                "rootFolderId": root_folder_id,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls, src_dict: Mapping[str, Any]) -> Self:
        d = dict(src_dict)
        id = d.pop("id")

        name = d.pop("name")

        library_type = LibraryType(d.pop("libraryType"))

        space_id = d.pop("spaceId")

        root_folder_id = d.pop("rootFolderId")

        library = cls(
            id=id,
            name=name,
            library_type=library_type,
            space_id=space_id,
            root_folder_id=root_folder_id,
        )

        return library



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from typing_extensions import Self

from templafy.types import UNSET, Unset

if TYPE_CHECKING:
    from templafy.models.create_image_field_schema_request import (
        CreateImageFieldSchemaRequest,
    )
    from templafy.models.create_number_field_schema_request import (
        CreateNumberFieldSchemaRequest,
    )
    from templafy.models.create_reference_field_schema_request import (
        CreateReferenceFieldSchemaRequest,
    )
    from templafy.models.create_text_field_schema_request import (
        CreateTextFieldSchemaRequest,
    )

# Runtime imports moved to top-level to satisfy PLC0415
from templafy.models.create_image_field_schema_request import (
    CreateImageFieldSchemaRequest,
)
from templafy.models.create_number_field_schema_request import (
    CreateNumberFieldSchemaRequest,
)
from templafy.models.create_reference_field_schema_request import (
    CreateReferenceFieldSchemaRequest,
)
from templafy.models.create_text_field_schema_request import (
    CreateTextFieldSchemaRequest,
)

T = TypeVar("T", bound="CreateDataSourceRequest")


@_attrs_define
class CreateDataSourceRequest:
    """Example:
        {'name': 'Cities', 'description': 'Cities in which we have offices', 'fields': [{'name': 'History', 'type':
            'text', 'isMultipleLines': True, 'defaultValue': 'The city was established in the year 1652 by Dutch
            explorers...', 'isRequired': False}, {'name': 'Population', 'type': 'number', 'defaultValue': 100222,
            'isRequired': True}, {'name': 'Country', 'type': 'reference', 'referenceDataSourceId': '637989101951089955',
            'defaultReferencedItemId': '638249311425155568', 'isRequired': True}, {'name': 'Flag', 'type': 'image',
            'isRequired': True}]}

    Attributes:
        name (str): The name of the data source. It must be unique. Max length is 100 characters.
        description (Union[None, Unset, str]): The description of the data source.
        fields (Union[None, Unset, list[Union['CreateImageFieldSchemaRequest', 'CreateNumberFieldSchemaRequest',
            'CreateReferenceFieldSchemaRequest', 'CreateTextFieldSchemaRequest']]]): The fields of the data source. If not
            specified, the data source will be created without fields.
    """

    name: str
    description: None | Unset | str = UNSET
    fields: (
        None
        | Unset
        | list[
            Union[
                "CreateImageFieldSchemaRequest",
                "CreateNumberFieldSchemaRequest",
                "CreateReferenceFieldSchemaRequest",
                "CreateTextFieldSchemaRequest",
            ]
        ]
    ) = UNSET

    def to_dict(self) -> dict[str, Any]:
        """Convert the object to a dictionary."""
        # Imports moved to module top-level to satisfy linter PLC0415

        name = self.name

        description: None | Unset | str
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description

        fields: None | Unset | list[dict[str, Any]]
        if isinstance(self.fields, Unset):
            fields = UNSET
        elif isinstance(self.fields, list):
            fields = []
            for fields_type_0_item_data in self.fields:
                fields_type_0_item: dict[str, Any]
                if isinstance(
                    fields_type_0_item_data,
                    (
                        CreateTextFieldSchemaRequest,
                        CreateNumberFieldSchemaRequest,
                        CreateImageFieldSchemaRequest,
                    ),
                ):
                    fields_type_0_item = fields_type_0_item_data.to_dict()
                else:
                    fields_type_0_item = fields_type_0_item_data.to_dict()

                fields.append(fields_type_0_item)

        else:
            fields = self.fields

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "name": name,
            }
        )
        if description is not UNSET:
            field_dict["description"] = description
        if fields is not UNSET:
            field_dict["fields"] = fields

        return field_dict

    @classmethod
    def from_dict(cls, src_dict: Mapping[str, Any]) -> Self:
        """Create an instance from a dictionary."""
        # Imports moved to module top-level to satisfy linter PLC0415

        d = dict(src_dict)
        name = d.pop("name")

        def _parse_description(data: object) -> None | Unset | str:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast("None | Unset | str", data)

        description = _parse_description(d.pop("description", UNSET))

        def _parse_fields(
            data: object,
        ) -> (
            None
            | Unset
            | list[
                Union[
                    "CreateImageFieldSchemaRequest",
                    "CreateNumberFieldSchemaRequest",
                    "CreateReferenceFieldSchemaRequest",
                    "CreateTextFieldSchemaRequest",
                ]
            ]
        ):
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError
                fields_type_0 = []
                _fields_type_0 = data
                for fields_type_0_item_data in _fields_type_0:

                    def _parse_fields_type_0_item(
                        data: object,
                    ) -> Union[
                        "CreateImageFieldSchemaRequest",
                        "CreateNumberFieldSchemaRequest",
                        "CreateReferenceFieldSchemaRequest",
                        "CreateTextFieldSchemaRequest",
                    ]:
                        try:
                            if not isinstance(data, dict):
                                raise TypeError
                            fields_type_0_item_type_0 = (
                                CreateTextFieldSchemaRequest.from_dict(data)
                            )

                            return fields_type_0_item_type_0
                        except:  # noqa: E722
                            pass
                        try:
                            if not isinstance(data, dict):
                                raise TypeError
                            fields_type_0_item_type_1 = (
                                CreateNumberFieldSchemaRequest.from_dict(data)
                            )

                            return fields_type_0_item_type_1
                        except:  # noqa: E722
                            pass
                        try:
                            if not isinstance(data, dict):
                                raise TypeError
                            fields_type_0_item_type_2 = (
                                CreateImageFieldSchemaRequest.from_dict(data)
                            )

                            return fields_type_0_item_type_2
                        except:  # noqa: E722
                            pass
                        if not isinstance(data, dict):
                            raise TypeError
                        fields_type_0_item_type_3 = (
                            CreateReferenceFieldSchemaRequest.from_dict(data)
                        )

                        return fields_type_0_item_type_3

                    fields_type_0_item = _parse_fields_type_0_item(
                        fields_type_0_item_data
                    )

                    fields_type_0.append(fields_type_0_item)

                return fields_type_0
            except:  # noqa: E722
                pass
            return cast(
                "None | Unset | list[CreateImageFieldSchemaRequest | CreateNumberFieldSchemaRequest | CreateReferenceFieldSchemaRequest | CreateTextFieldSchemaRequest]",
                data,
            )

        fields = _parse_fields(d.pop("fields", UNSET))

        create_data_source_request = cls(
            name=name,
            description=description,
            fields=fields,
        )

        return create_data_source_request



from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Self

T = TypeVar("T", bound="UpdateNumberDataSourceItemFieldRequest")


@_attrs_define
class UpdateNumberDataSourceItemFieldRequest:
    """Example:
        {'type': 'number', 'value': 123.45}

    Attributes:
        type_ (str): Data source item field type.
        value (float): The value of the field with the precision of 2 decimal places.
    """

    type_: str
    value: float
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        type_ = self.type_

        value = self.value

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type_,
                "value": value,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls, src_dict: Mapping[str, Any]) -> Self:
        d = dict(src_dict)
        type_ = d.pop("type")

        value = d.pop("value")

        update_number_data_source_item_field_request = cls(
            type_=type_,
            value=value,
        )

        update_number_data_source_item_field_request.additional_properties = d
        return update_number_data_source_item_field_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
